import { historyField, defaultKeymap, indentWithTab } from '@codemirror/commands';
import { indentUnit } from '@codemirror/language';
import { Compartment, EditorState } from '@codemirror/state';
import { EditorView, keymap } from '@codemirror/view';
import { map } from 'nanostores';

// src/index.ts
var withCodemirrorInstance = () => map({
  view: null,
  extensions: null,
  value: null,
  documents: /* @__PURE__ */ new Map()
});
var codemirror = (node, options) => {
  if (is_nullish(options))
    throw new Error("No options provided. At least `value` is required.");
  let { value, instanceStore, onChangeBehavior = { kind: "debounce", duration: 50 } } = options;
  const EDITOR_STATE_MAP = /* @__PURE__ */ new Map();
  let fulfill_editor_initialized;
  let editor_initialized = new Promise((r) => fulfill_editor_initialized = r);
  let view;
  let internal_extensions = [];
  const setup_compartment = new Compartment();
  const lang_compartment = new Compartment();
  const theming_compartment = new Compartment();
  const tabs_compartment = new Compartment();
  const readonly_compartment = new Compartment();
  const extensions_compartment = new Compartment();
  const autocomplete_compartment = new Compartment();
  const linter_compartment = new Compartment();
  const watcher = EditorView.updateListener.of((view_update) => on_change(view_update));
  async function make_extensions(options2) {
    return Promise.all([
      watcher,
      // User extensions matter the most, keep em on top
      extensions_compartment.of(get_user_extensions(options2)),
      // Autocomplete may come built in with setup: basic, so always keep it above setup_compartment
      autocomplete_compartment.of(await get_autocompletion(options2)),
      setup_compartment.of(await get_setup(options2)),
      // Needs to be under `setup` because setup, if there, will add the indentWithTab
      keymap.of([...defaultKeymap, ...options2.useTabs ? [indentWithTab] : []]),
      lang_compartment.of(await get_lang(options2)),
      theming_compartment.of(get_theme(options2)),
      tabs_compartment.of(await get_tab_setting(options2)),
      readonly_compartment.of(get_readonly(options2)),
      linter_compartment.of(await get_linter(options2))
    ]);
  }
  function dispatch_event(event, detail) {
    node.dispatchEvent(new CustomEvent(event, detail ? { detail } : void 0));
  }
  function handle_change(view_update) {
    const new_value = view.state.doc.toString();
    if (!is_equal(new_value, value)) {
      value = new_value;
      dispatch_event("codemirror:textChange", value);
    }
    instanceStore?.set({
      value,
      view,
      extensions: internal_extensions,
      documents: EDITOR_STATE_MAP
    });
    dispatch_event("codemirror:change", view_update);
  }
  const { kind: behaviorKind = "debounce", duration: behaviorDuration = 50 } = is_nullish(
    onChangeBehavior
  ) ? {} : onChangeBehavior;
  let on_change = behaviorKind === "debounce" ? debounce(handle_change, behaviorDuration) : throttle(handle_change, behaviorDuration);
  (async () => {
    internal_extensions = await make_extensions(options);
    const state = EditorState.create({
      doc: value,
      extensions: internal_extensions,
      selection: {
        anchor: options.cursorPos ?? 0,
        head: options.cursorPos ?? 0
      }
    });
    view = new EditorView({
      state,
      parent: node
    });
    if (!is_nullish(options.cursorPos))
      view.focus();
    fulfill_editor_initialized();
  })();
  return {
    async update(new_options) {
      await editor_initialized;
      const transaction = {};
      if (!is_equal(value, new_options.value)) {
        transaction.changes = {
          from: 0,
          to: view.state.doc.length,
          insert: new_options.value
        };
      }
      if (!is_nullish(new_options.cursorPos) && !is_equal(options.cursorPos, new_options.cursorPos)) {
        console.log("setting cursorPos", new_options.cursorPos);
        transaction.selection = {
          anchor: new_options.cursorPos ?? 0,
          head: new_options.cursorPos ?? 0
        };
        view.focus();
      }
      async function append_effect(compartment, options_list, factory) {
        transaction.effects = transaction.effects ?? [];
        const effects = transaction.effects;
        let are_all_options_nullish = true;
        for (const option_name of options_list) {
          const new_option = new_options[option_name];
          const old_option = options[option_name];
          if (!is_nullish(new_option)) {
            are_all_options_nullish = false;
            if (!is_equal(new_option, old_option)) {
              return effects.push(compartment.reconfigure(await factory(new_options)));
            }
          }
        }
        if (are_all_options_nullish)
          effects.push(compartment.reconfigure([]));
      }
      const pre_transaction_state = view.state.toJSON(
        is_object(new_options.documentFields) ? new_options.documentFields : DEFAULT_DOCUMENT_FIELDS
      );
      const internal_extensions_promise = make_extensions(new_options);
      if (is_equal(options.documentId, new_options.documentId) && !is_nullish(options.documentId)) {
        console.log(1);
        await Promise.all([
          append_effect(setup_compartment, ["setup"], get_setup),
          append_effect(lang_compartment, ["lang"], get_lang),
          append_effect(tabs_compartment, ["useTabs", "tabSize"], get_tab_setting),
          append_effect(theming_compartment, ["theme"], get_theme),
          append_effect(extensions_compartment, ["extensions"], get_user_extensions),
          append_effect(readonly_compartment, ["readonly"], get_readonly),
          append_effect(autocomplete_compartment, ["autocomplete"], get_autocompletion),
          append_effect(linter_compartment, ["lint", "lintOptions"], get_linter)
        ]);
        view.dispatch(transaction);
        internal_extensions = await internal_extensions_promise;
      }
      if (!is_nullish(options.documentId) && !is_equal(options.documentId, new_options.documentId)) {
        EDITOR_STATE_MAP.set(options.documentId, pre_transaction_state);
        if (!is_nullish(new_options.documentId)) {
          const old_state = EDITOR_STATE_MAP.get(new_options.documentId);
          dispatch_event("codemirror:documentChanging", { view });
          internal_extensions = await internal_extensions_promise;
          view.setState(
            old_state ? EditorState.fromJSON(
              old_state,
              { extensions: internal_extensions, doc: new_options.value },
              is_object(new_options.documentFields) ? new_options.documentFields : DEFAULT_DOCUMENT_FIELDS
            ) : EditorState.create({
              doc: new_options.value,
              extensions: internal_extensions
            })
          );
          view.focus();
          dispatch_event("codemirror:documentChanged", { view });
        }
      }
      const { kind: behaviorKind2 = "debounce", duration: behaviorDuration2 = 50 } = new_options.onChangeBehavior ?? { kind: "debounce", duration: 50 };
      if (!is_equal(options.onChangeBehavior, new_options.onChangeBehavior)) {
        on_change = behaviorKind2 === "debounce" ? debounce(handle_change, behaviorDuration2) : throttle(handle_change, behaviorDuration2);
      }
      options = new_options;
      internal_extensions = await make_extensions(new_options);
    },
    destroy() {
      editor_initialized.then(() => view?.destroy());
    }
  };
};
var DEFAULT_DOCUMENT_FIELDS = {
  history: historyField
};
async function get_setup(options) {
  const { setup } = options;
  if (is_nullish(setup))
    return [];
  if (setup === "basic")
    return (await import('./basic-setup-BJSL5TOQ.js')).default(options);
  if (setup === "minimal")
    return (await import('./minimal-setup-Y523JPZT.js')).default(options);
  throw new Error(
    "`setup` can only be `basic` or `minimal`. If you wish to provide another setup, pass through `extensions` prop."
  );
}
async function get_lang({ lang, langMap }) {
  if (is_nullish(lang))
    return [];
  if (typeof lang === "string") {
    if (!langMap)
      throw new Error("`langMap` is required when `lang` is a string.");
    if (!(lang in langMap))
      throw new Error(`Language "${lang}" is not defined in \`langMap\`.`);
    const lang_support = await langMap[lang]();
    return lang_support;
  }
  return lang;
}
function get_theme({ theme, styles }) {
  return [theme, styles && EditorView.theme(styles)].filter(Boolean);
}
async function get_tab_setting({ useTabs = false, tabSize = 2 }) {
  if (is_nullish(tabSize))
    return [];
  return [EditorState.tabSize.of(tabSize), indentUnit.of(useTabs ? "	" : " ".repeat(tabSize))];
}
async function get_autocompletion({ autocomplete }) {
  if (is_nullish(autocomplete))
    return [];
  const { autocompletion } = await import('@codemirror/autocomplete');
  return autocompletion(typeof autocomplete === "object" && autocomplete ? autocomplete : {});
}
function get_readonly({ readonly }) {
  return EditorState.readOnly.of(!!readonly);
}
function get_user_extensions({ extensions }) {
  return extensions ?? [];
}
async function get_linter({ lint, lintOptions = {} }) {
  if (is_nullish(lint))
    return [];
  if (!is_function(lint))
    throw new Error("`lint` must be a function.");
  const { linter } = await import('@codemirror/lint');
  return linter(lint, lintOptions ?? {});
}
var is_equal = (a, b) => {
  if (!is_object(a) || !is_object(b))
    return a === b;
  const sortedStr = (obj) => JSON.stringify(obj, Object.keys(obj).sort());
  try {
    return sortedStr(a) === sortedStr(b);
  } catch {
    return false;
  }
};
var is_nullish = (a) => a == null;
var is_function = (a) => typeof a === "function";
var is_object = (a) => !is_nullish(a) && typeof a === "object";
function debounce(func, threshold, execAsap = false) {
  let timeout;
  return function debounced(...args) {
    const self = this;
    if (timeout)
      clearTimeout(timeout);
    else if (execAsap)
      func.apply(self, args);
    timeout = setTimeout(delayed, threshold || 100);
    function delayed() {
      if (!execAsap)
        func.apply(self, args);
      timeout = null;
    }
  };
}
function throttle(func, threshold) {
  let last_args;
  let should_wait = false;
  function timeout_function(self) {
    if (last_args) {
      func.apply(self, last_args);
      setTimeout(timeout_function, threshold, self);
      last_args = null;
      return;
    }
    should_wait = false;
  }
  return function throttled(...args) {
    const self = this;
    if (should_wait) {
      last_args = args;
      return;
    }
    func.apply(self, args);
    should_wait = true;
    setTimeout(timeout_function, threshold, self);
  };
}

export { codemirror, withCodemirrorInstance };
